﻿
Imports System.Math
Imports System.Drawing.Imaging

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False


    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function


    ' =======================================================================================================
    '   Files
    ' =======================================================================================================

    ' returns lower-case extension with initial dot
    Public Function GetExtension(ByVal str As String) As String
        On Error Resume Next
        Return LCase(IO.Path.GetExtension(str))
    End Function

    Public Function RemoveExtension(ByVal str As String) As String
        On Error Resume Next
        Return PlatformAdjustedFileName(IO.Path.GetDirectoryName(str) & "\" & IO.Path.GetFileNameWithoutExtension(str))
    End Function

    Public Function FileExists(ByVal fileName As String) As Boolean
        Return My.Computer.FileSystem.FileExists(fileName)
    End Function

    Public Function FolderExists(ByVal FolderName As String) As Boolean
        If FolderName.Length < 2 Then Return False
        FolderName = LCase(FolderName)
        Select Case FolderName
            Case "a:\", "b:\", "c:\", "d:\", "e:\", "f:\", "g:\", "h:\", "i:\", "j:\", "k:\"
                Return True
        End Select
        Return My.Computer.FileSystem.DirectoryExists(FolderName)
    End Function


    ' =======================================================================================================
    '   Platform selection
    ' =======================================================================================================
    'Friend CanUseWindowsDlls As Boolean = _CanUseWindowsDlls()
    Friend Function CanUseWindowsDlls() As Boolean
        Select Case Environment.OSVersion.Platform
            Case PlatformID.Win32NT, PlatformID.Win32S, PlatformID.Win32Windows, PlatformID.WinCE
                Return True
            Case Else
                Return False
        End Select
    End Function
    Friend Function PlatformAdjustedFileName(ByVal FileName As String, _
                                             Optional ByVal DefaultName As String = "") As String
        If CanUseWindowsDlls Then
            FileName = FileName.Replace("/", "\")
            Return FileName
        Else
            If FileName.Contains(":") Then FileName = DefaultName
            FileName = FileName.Replace("\", "/")
            Return FileName
        End If
    End Function

    ' =======================================================================================
    '  FORM
    ' =======================================================================================

    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Private sw As Int32
    Private sh As Int32
    Private Sub GetMaxScreenBounds()
        ' -------------------------------------- Gets an array of all the screens connected to the system.
        Dim Screens() As System.Windows.Forms.Screen = System.Windows.Forms.Screen.AllScreens
        ' -------------------------------------- For each screen
        Dim Index As Int32
        For Index = 0 To Screens.GetUpperBound(0)
            sw = Max(sw, Screens(Index).Bounds.Left + Screens(Index).Bounds.Width)
            sh = Max(sh, Screens(Index).Bounds.Top + Screens(Index).Bounds.Height)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, sw)           ' non oltre le dimensioni di una maximized window
    '        .Height = Math.Min(.Height, sh)         ' non oltre le dimensioni di una maximized window
    '        .Width = Math.Max(.Width, 32)           ' non meno di 32x24
    '        .Height = Math.Max(.Height, 24)         ' non meno di 32x24
    '        .Left = Math.Min(.Left, sw - .Width)    ' non oltre il bordo destro
    '        .Top = Math.Min(.Top, sh - .Height)     ' non oltre il bordo inferiore
    '        .Left = Math.Max(.Left, 0)              ' non meno del bordo sinistro
    '        .Top = Math.Max(.Top, 0)                ' non sopra al bordo superiore
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByRef frm As Form)
        With frm
            .Width = Math.Min(.Width, sw)               ' non oltre le dimensioni del VIRTUALSCREEN
            .Height = Math.Min(.Height, sh)
            .Width = Math.Max(.Width, 32)               ' non meno di 32x24
            .Height = Math.Max(.Height, 24)             ' non meno di 32x24
            .Left = Math.Min(.Left, sw - 50)            ' non oltre il bordo destro - 50
            .Top = Math.Min(.Top, sh - 50)              ' non oltre il bordo inferiore - 50
            .Left = Math.Max(.Left, 50 - .Width)        ' non meno del bordo sinistro + 50
            .Top = Math.Max(.Top, -20)                  ' non sopra al bordo superiore
        End With
    End Sub


    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================

    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function

    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function

    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Double(ByVal l As String) As Double
        Return Val(Replace(l, ",", "."))
    End Function

    Private Function MultilineToString(ByVal Name As String, _
                                  ByVal Value As String) As String

        Value = Value.Replace(vbCrLf, "%M")
        Return Name & "=" & Value
    End Function

    Private Function StringToMultiline(ByVal Value As String) As String
        Value = Value.Replace("%M", vbCrLf)
        Return Value
    End Function


    Friend Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Int32
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function



    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub SaveConfigurationAs()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.DefaultExt = ".txt"
        sfd.Filter = "Configuration file (*.txt)|*txt"
        'sfd.FileName = AssemblyName() & "_INI_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        sfd.FileName = "MCA_INI_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Save_INI(sfd.FileName)
        End If
    End Sub

    Friend Sub LoadConfiguration()
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.Filter = "Configuration file (*.txt)|*txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Load_INI(ofd.FileName)
        End If
    End Sub


    Friend Sub Save_INI(Optional ByVal iniFileName As String = "")
        '
        Try
            '
            If iniFileName = "" Then
                iniFileName = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_INI.txt")
            End If
            '
            Dim f As System.IO.StreamWriter
            f = IO.File.CreateText(iniFileName)
            '
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form_Main)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form_Main.WindowState))
            '
            '
            'f.WriteLine("")
            'f.WriteLine(" Params")
            'f.WriteLine("===========================================")
            f.WriteLine(TabString("PortNumberIndex", Form_Main.cmb_PortNumber.SelectedIndex.ToString))
            f.WriteLine(TabString("BaudRateIndex", Form_Main.cmb_BaudRate.SelectedIndex.ToString))
            f.WriteLine(TabString("Slot", Form_Main.txt_Slot.Text))
            f.WriteLine(TabString("Protocol", Form_Main.cmb_Protocol.SelectedIndex.ToString))
            'f.WriteLine(TabString("DeadTime", Form_Main.txt_DeadTime.NumericValueInteger.ToString))
            'f.WriteLine(TabString("IIR_Filter", Form_Main.txt_IIR_Filter.NumericValueInteger.ToString))
            ''
            'f.WriteLine("")
            'f.WriteLine("===========================================")
            'f.WriteLine(" BaseLine")
            'f.WriteLine(TabString("BaseLineEnable", Form_Main.chk_BlrEnable.Checked))
            'f.WriteLine(TabString("BaseLinePosition", Form_Main.txt_Blr_Position.NumericValueInteger))
            'f.WriteLine(TabString("BaseLineSize", Form_Main.txt_Blr_Size.NumericValueInteger))
            '
            f.Close()
        Catch
        End Try
    End Sub





    Friend Sub Load_INI(Optional ByVal iniFileName As String = "")
        ' ----------------------------------------- avoid trackbars and other controls min-max overflow
        On Error Resume Next
        ' 
        Dim LoadAll As Boolean = False
        If iniFileName = "" Then
            iniFileName = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
            LoadAll = True
        End If
        '
        Dim l As String
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Dim param As String = ExtractParamName(l)
                Select Case param
                    ' -----------------------------------------------------------------
                    Case "Form1_Top" : If LoadAll Then Form_Main.Top = CInt(Val(l))
                    Case "Form1_Left" : If LoadAll Then Form_Main.Left = CInt(Val(l))
                    Case "Form1_Width" : If LoadAll Then Form_Main.Width = CInt(Val(l))
                    Case "Form1_Height" : If LoadAll Then Form_Main.Height = CInt(Val(l))

                        '    ' ----------------------------------------------------------------- Params
                    Case "PortNumberIndex" : Form_Main.cmb_PortNumber.SelectedIndex = CInt(Val(l))
                    Case "BaudRateIndex" : Form_Main.cmb_BaudRate.SelectedIndex = CInt(Val(l))
                    Case "Slot" : Form_Main.txt_Slot.Text = l
                    Case "RadProtocol" : Form_Main.cmb_Protocol.SelectedIndex = CInt(Val(l))
                        'Case "DeadTime" : Form_Main.txt_DeadTime.NumericValueInteger = CInt(Val(l))
                        'Case "IIR_Filter" : Form_Main.txt_IIR_Filter.NumericValueInteger = CInt(Val(l))
                        '    ' ----------------------------------------------------------------- BaseLine
                        'Case "BaseLineEnable" : Form_Main.chk_BlrEnable.Checked = l = "True"
                        'Case "BaseLinePosition" : Form_Main.txt_Blr_Position.Text = l
                        'Case "BaseLineSize" : Form_Main.txt_Blr_Size.Text = l
                End Select
            Loop
            f.Close()
        End If
        If Form_Main.WindowState = FormWindowState.Minimized Then Form_Main.WindowState = FormWindowState.Normal
        LimitFormPosition(Form_Main)
    End Sub


    ' ==================================================================================================
    '  SAVE IMAGE
    ' ==================================================================================================
    Public Sub SaveImage(ByVal img As Image, _
                         ByVal filename As String, _
                         ByVal extension As String, _
                         ByVal Quality As Int32)

        extension = LCase(extension)
        filename = RemoveExtension(filename)
        filename += "." & extension
        '
        If img Is Nothing Then Exit Sub
        Try
            File_Kill(filename)
            If extension = "jpg" Then
                Dim ImageEncoders() As ImageCodecInfo = ImageCodecInfo.GetImageEncoders()
                Dim myEncoder As System.Drawing.Imaging.Encoder = System.Drawing.Imaging.Encoder.Quality
                Dim myEncoderParameters As New EncoderParameters(1)
                Dim myEncoderParameter As New EncoderParameter(myEncoder, 100)
                myEncoderParameters.Param(0) = myEncoderParameter
                img.Save(filename, ImageEncoders(1), myEncoderParameters)
            Else
                img.Save(filename, ImageFormatFromFileExtension(extension))
            End If
        Catch
            MsgBox("Image save error", MsgBoxStyle.Exclamation)
        End Try
    End Sub

    Private Function ImageFormatFromFileExtension(ByVal extension As String) As ImageFormat
        Select Case LCase(extension)
            Case "jpg" : Return ImageFormat.Jpeg
            Case "png" : Return ImageFormat.Png
            Case "tiff" : Return ImageFormat.Tiff
            Case "exif" : Return ImageFormat.Exif
            Case "emf" : Return ImageFormat.Emf
            Case "wmf" : Return ImageFormat.Wmf
            Case "gif" : Return ImageFormat.Gif
            Case "bmp" : Return ImageFormat.Bmp
                'Case "ico" : Return ImageFormat.Icon
            Case Else : Return ImageFormat.Jpeg
        End Select
    End Function

    Friend Sub File_Kill(ByVal filename As String)
        If My.Computer.FileSystem.FileExists(filename) Then
            My.Computer.FileSystem.DeleteFile(filename, FileIO.UIOption.OnlyErrorDialogs, FileIO.RecycleOption.DeletePermanently)
        End If
    End Sub


    Friend Sub SaveImage(ByVal Prefix As String, ByVal cnt As Control)
        Dim img As Image = GetImage(cnt)
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.Filter = "Portable network graphics (*.png)|*png|JPEG image (*.jpg)|*.jpg"
        sfd.FileName = Prefix & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SaveImage(img, sfd.FileName, If(sfd.FilterIndex = 2, "jpg", "png"), 100)
        End If
    End Sub
    Friend Function GetImage(ByVal cnt As Control) As Bitmap
        Dim s As Size = cnt.Size
        Dim bmp As Bitmap = New Bitmap(s.Width, s.Height)
        cnt.DrawToBitmap(bmp, New Rectangle(0, 0, s.Width, s.Height))
        Return bmp
    End Function


    ' ================================================================================================
    '  Misc Utils
    ' ================================================================================================

    Friend Function MouseButtonLeftPressed() As Boolean
        Return (Control.MouseButtons And Windows.Forms.MouseButtons.Left) <> Windows.Forms.MouseButtons.None
    End Function

End Module
